@@ -8,6 +8,9 @@ Config.define('WORKER_SLEEP_TIME', 10, 'Main loop sleep time', 'Worker')
 Config.define('ZOMBIE_WORKER_TIME', 200,
               'Time to remove a Worker from API List (must be greater than WORKER_SLEEP_TIME + Validation time)', 'API')
 
+Config.define('CONNECT_TIMEOUT_IN_SECONDS', 20, 'Number of seconds a connection can take.', 'Worker')
+Config.define('REQUEST_TIMEOUT_IN_SECONDS', 60, 'Number of seconds a request can take.', 'Worker')
+
 Config.define('HOLMES_API_URL', 'http://localhost:2368', 'URL that Worker will communicate with API', 'Worker')
 
 Config.define('LOG_LEVEL', 'ERROR', 'Default log level', 'Logging')
@@ -16,6 +19,7 @@ Config.define('LOG_FORMAT', '%(asctime)s:%(levelname)s %(module)s - %(message)s'
 Config.define('LOG_DATE_FORMAT', '%Y-%m-%d %H:%M:%S',
               'Date Format to be used when writing log messages.', 'Logging')
 
+Config.define('FACTERS', [], 'List of classes to get facts about a website', 'Review')
 Config.define('VALIDATORS', [], 'List of classes to validate a website', 'Review')
 Config.define('REVIEW_EXPIRATION_IN_SECONDS', 6 * 60 * 60, 'Number of seconds that a review expires in.', 'Review')
 
@@ -48,3 +52,9 @@ Config.define('HTTP_PROXY_HOST', None, 'HTTP Proxy Host to use', 'Web')
 Config.define('HTTP_PROXY_PORT', None, 'HTTP Proxy Port to use', 'Web')
 
 Config.define('COMMIT_ON_REQUEST_END', True, 'Commit on request end', 'DB')
+
+Config.define('REDISHOST', 'localhost', 'Redis host', 'Redis')
+Config.define('REDISPORT', 7575, 'Redis port', 'Redis')
+Config.define('REDISPASS', None, 'Redis password in case of auth', 'Redis')
+
+Config.define('REQUIRED_META_TAGS', [], 'List of required meta tags', 'Meta tag Validator')
