@@ -2,8 +2,9 @@
 # -*- coding: utf-8 -*-
 
 from uuid import UUID
+import hashlib
 
-from ujson import loads
+from ujson import loads, dumps
 from tornado import gen
 import tornado.httpclient
 #from motorengine import Q, DESCENDING
@@ -44,6 +45,7 @@ class PageHandler(BaseHandler):
         logging.info('Obtaining "%s" using proxy "%s:%s"...' % (url, phost, pport))
 
         response = yield tornado.gen.Task(self.application.http_client.fetch, request)
+
         if response.code > 399:
             self.set_status(400, 'Invalid URL [%s]' % url)
             self.write_json({
@@ -75,10 +77,16 @@ class PageHandler(BaseHandler):
             domain = domains[0]
 
         if not domain:
-            domain = Domain(url=domain_url, name=domain_name)
+            url_hash = hashlib.sha512(domain_url).hexdigest()
+            domain = Domain(url=domain_url, url_hash=url_hash, name=domain_name)
             self.db.add(domain)
             self.db.flush()
 
+            self.application.event_bus.publish(dumps({
+                'type': 'new-domain',
+                'domainUrl': str(domain_url)
+            }))
+
         pages = self.db.query(Page).filter(or_(
             Page.url == url,
             Page.url == url.rstrip('/'),
@@ -95,10 +103,17 @@ class PageHandler(BaseHandler):
             self.finish()
             return
 
-        page = Page(url=url, domain=domain)
+        url_hash = hashlib.sha512(url).hexdigest()
+
+        page = Page(url=url, url_hash=url_hash, domain=domain)
         self.db.add(page)
         self.db.flush()
 
+        self.application.event_bus.publish(dumps({
+            'type': 'new-page',
+            'pageUrl': str(url)
+        }))
+
         self.write(str(page.uuid))
         self.finish()
 
@@ -164,56 +179,107 @@ class PagesHandler(BaseHandler):
             self.finish()
             return
 
-        pages_to_add = []
-
         all_domains = []
         for url in urls:
-            domain_name, domain_url = get_domain_from_url(url)
+            domain_name, domain_url = get_domain_from_url(url.strip())
             if not domain_name:
                 self.set_status(400, 'In the urls you posted there is an invalid URL: %s' % url)
                 self.finish()
                 return
             all_domains.append((domain_name, domain_url))
 
-        existing_domains = self.db.query(Domain).filter(Domain.name.in_([domain[0] for domain in all_domains])).all()
-        existing_domains_dict = dict([(domain.name, domain) for domain in existing_domains])
+        domains = self.add_domains(all_domains)
+        pages = self.add_pages(urls, domains)
 
-        domains_to_add = [
-            Domain(name=domain[0], url=domain[1])
-            for domain in all_domains
-            if not domain[0] in existing_domains_dict
-        ]
-        domains_to_add_dict = dict([(domain.name, domain) for domain in domains_to_add])
+        self.write(str(len(pages)))
+        self.finish()
 
-        all_domains_dict = {}
-        all_domains_dict.update(existing_domains_dict)
-        all_domains_dict.update(domains_to_add_dict)
+    def add_domains(self, domains):
+        resulting_domains = {}
 
+        #existing_domains = self.db.query(Domain.id, Domain.url).filter(
+            #or_(
+                #Domain.url.in_([
+                    #domain[1] for domain in domains
+                #]),
+                #Domain.url.in_([
+                    #domain[1].rstrip('/') for domain in domains
+                #]),
+                #Domain.url.in_([
+                    #"%s/" % domain[1] for domain in domains
+                #])
+            #)
+        #).all()
+        #existing_domain_dict = dict([(domain[0], domain[1]) for domain in existing_domains])
+
+        for domain_name, domain_url in domains:
+            if domain_url in resulting_domains:
+                continue
+
+            domain = self.db.query(Domain).filter(or_(
+                Domain.url == domain_url,
+                Domain.url == domain_url.rstrip('/')
+            )).first()
+
+            if domain:
+                resulting_domains[domain_url] = domain
+                continue
+
+            key = domain_url.rstrip('/')
+            domain = self.db.query(Domain).filter(Domain.url == key).first()
+            if domain:
+                resulting_domains[domain_url] = domain
+                continue
+
+            key = ("%s/" % domain_url)
+            domain = self.db.query(Domain).filter(Domain.url == key).first()
+            if domain:
+                resulting_domains[domain_url] = domain
+                continue
+
+            resulting_domains[domain_url] = Domain(
+                name=domain_name,
+                url=domain_url,
+                url_hash=hashlib.sha512(domain_url).hexdigest())
+
+            self.db.add(resulting_domains[domain_url])
+
+            self.application.event_bus.publish(dumps({
+                'type': 'new-domain',
+                'domainUrl': domain_url
+            }))
+
+        self.db.flush()
+        return resulting_domains
+
+    def add_pages(self, urls, domains):
         existing_pages = self.db.query(Page).filter(Page.url.in_(urls)).all()
         if not existing_pages:
             existing_pages = []
         existing_pages_dict = dict([(page.url, page) for page in existing_pages])
 
-        pages_to_add = []
-        for url in urls:
+
+        added_pages = []
+        for url in set(urls):
             if url in existing_pages_dict:
                 continue
             domain_name, domain_url = get_domain_from_url(url.strip())
-            domain = all_domains_dict[domain_name]
 
             logging.debug("Adding URL: %s" % url)
-            pages_to_add.append(Page(url=url, domain=domain))
+            url_hash = hashlib.sha512(str(url)).hexdigest()
 
-        if domains_to_add:
-            for domain in domains_to_add:
-                self.db.add(domain)
+            page = Page(url=str(url), url_hash=url_hash, domain=domains[domain_url])
+            self.db.add(page)
+            added_pages.append(page)
 
-        if pages_to_add:
-            for page in pages_to_add:
-                self.db.add(page)
+        if added_pages:
+            self.application.event_bus.publish(dumps({
+                'type': 'new-page',
+                'pageUrl': added_pages[0].url
+            }))
 
-        self.write(str(len(pages_to_add)))
-        self.finish()
+        self.db.flush()
+        return added_pages
 
 
 class PageViolationsPerDayHandler(BaseHandler):
